CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  DISPINV.FOR                                                       C
C                                                                    C
C  by: Paul Segall and Laurie Erickson                               C
C  revised: February 9, 1988                                         C
C  revised: July 15, 1988; added studentized residuals               C
C  revised: Sept 1, 1988; changed to new gstalc                      C
C  revised: Jan 5, 1989; shrinks G to exclude stations with no data  C
C                                                                    C
C        Program DISPINV.FOR solves for benchmark displacements      C
C  (velocities) given mark-to-mark baseline length changes (rates of C
C  change).  For general discussion of method see P.Segall and M.V.  C
C  Matthews, 1988; Displacement Calculations From Geodetic Data and  C
C  the Testing of Geophysical Deformation Models                     C
C        The general problem is:                                     C
C                                  d = G u + epsilon                 C
C                                                                    C
C  numst   = number of stations                                      C
C  numdat  = number of data                                          C
C  d       = data vector (length numdat, units = meters or meters/yr)C
C  llcsn   = array of pairs of baseline endpoint names               C
C            llcsn(2,numst)                                          C
C  G       = Green's function matrix (numdat x 2*numst)              C
C  epsilon = error vector; N(0, sigma)                               C
C            sigma is usually diagonal with variances = stdev**2     C
C  stdev   = vector of standard deviations                           C
C  td      = data weight matrix                                      C
C  tdinv   = inverse of data weight matrix                           C
C                                                                    C
C  SOLUTION TYPES:                                                   C
C  uinn    = inner coordinat solution                                C
C  uout    = outer coordinat solution                                C
C  umod    = model coordinat solution                                C
C            Note: displacement vectors are stored as:               C
C                  u(1) = first station, east component              C
C                  u(2) = first station, north component             C
C                  u(3) = second station, east component             C
C            Units = meters or meters/yr; length = 2*numst           C
C  sta     = char array of station names                             C
C  x,y     = x and y coordinates for all stations                    C
C  staloc  = array holding latitude and longitude of stations        C
C                                                                    C
C  VARIABLES:                                                        C
C  alpha   = angle in degrees between x and x-prime; measured        C
C            counter-clockwise  (used for outer coord solution)      C
C  uapr    = prior model for model coord solution (2*numst)          C
C                                                                    C
C  CONSTANTS:                                                        C
C  The following constants are used for dimensioning the arrays.     C
C  Their values must be specified in the Parameter statement.        C
C  ndim   =  the number of data observations                         C
C  mdim   =  the number of model parameters = 2*numst                C
C                                                                    C
C  The following variables are assigned constant values in the main  C
C  program.                                                          C
C  uot     =  unit number for general output file                    C
C  ust     =  unit number for station coordinate file                C
C  udt     =  unit number for data input files                       C
C  upm     =  unit number for prior model file                       C
C  uplt(1) =  unit number for inner coord plot file (output file)    C
C  uplt(2) =  unit number for outer coord plot file (output file)    C
C  uplt(3) =  unit number for model coord plot file (output file)    C
C  ustin   =  unit number for standard input (terminal)              C
C  ustout  =  unit number for standard output (terminal)             C
C  tol     =  tolerance in deciding when a singular value is zero    C
C  solopt(1) = option for inner coordinate solution                  C
C  solopt(2) = option for outer coordinate solution                  C
C  solopt(3) = option for model coordinate solution                  C
C  plotnm    = array holding output filenames for the inner, outer,  C
C              and model coord solutions respectively                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  MAIN PROGRAM
C
C THE USER MUST SET THE ARRAY DIMENSIONS IN THE FOLLOWING
C PARAMETER STATEMENTS
C
      Integer mdim, ndim, wkdim
      Parameter (mdim=500,ndim=500,wkdim=1000)
C
      Integer i,j,n,m,p
      Integer uot, ust, udt, upm, ustin, ustout, uplt(3)
      Integer numst, nmstus, numdat
      Real  x(mdim), y(mdim), az(ndim), staloc(mdim,4)
      Real smaj(mdim),smin(mdim),psi(mdim)
      Real stdev(ndim), td(ndim,ndim), tdinv(ndim,ndim)
      Real res(ndim),gpr(ndim,mdim),dpr(ndim),dcalc(ndim)
      Real tol,alpha, rpr(ndim), srpr(ndim)
      Real g(ndim,mdim),ut(ndim,ndim),s(mdim),v(mdim,mdim)
      Real ginv(mdim,ndim),d(ndim),sgn(mdim),uinn(mdim)
      Real qinn(mdim,mdim),wkar(wkdim,wkdim)
      Real v0(mdim,mdim),uout(mdim),qout(mdim,mdim)
      Real wk(wkdim),sm1(mdim),sm1upt(mdim,ndim)
      Real umod(mdim),qmod(mdim,mdim),uapr(mdim),qapr(mdim,mdim)
      real xorg, yorg
      Character*8 sta(mdim), llcsn(2,ndim)
      Character*80 plotnm(3)
      Character*1 solopt(3), used(mdim)
      Character*70 header
      uot     = 10
      ust     = 11
      udt     = 12
      upm     = 13
      uplt(1) = 14
      uplt(2) = 15
      uplt(3) = 16
      ustin   = 5
      ustout  = 6
      tol = 0.0001
 

C set-up.....
      Call Ginput(uot,upm,uplt,mdim,uapr,alpha,solopt,plotnm)
      xorg = 0.0
      yorg = 0.0
      Call Gstalc(uot,ust,mdim,x,y,sta,numst,staloc,xorg,yorg)
      Call Gdata(uot,udt,ndim,numdat,llcsn,stdev,d)

      if(2*numst.gt.mdim) then
        print *,'number of unknowns (m) exceeds array dimension mdim'
        print *,'increase mdim in paramter statement'
        stop
      endif

      Call Datwgt(ndim,stdev,numdat,td,tdinv)
      Call Makeg(ustout,mdim,ndim,numst,numdat,used,
     +                                     sta,llcsn,x,y,az,G)
      Call shrinkg(mdim,ndim,numst,nmstus,numdat,sta,used,staloc,G)
      m=2*nmstus
      n=numdat

      write(uot,*) ' '
      write(uot,*) ' '
      write(uot,*) numst-nmstus, '   STATIONS WITHOUT DATA EXCLUDED'
      do i=nmstus+1, numst
        write(uot,*) sta(i)
      end do

      Call wtgd(td,g,d,n,m,ndim,mdim,gpr,dpr)

C get singular value decomp.....
      call svd(gpr,n,m,ndim,mdim,ut,s,v,wk)
      header ='Singular Values'
      Call Prtvec(uot,mdim,m,header,s)
      tol = tol*s(1)
      call getp(s,n,m,mdim,tol,p)

      Write(uot,*) 'number of data                    = ',n
      Write(uot,*) 'number of unknowns                = ',m
      Write(uot,*) 'number of nonzero singular values = ',p
      Write(ustout,*) 'number of data                    = ',n
      Write(ustout,*) 'number of unknowns                = ',m
      Write(ustout,*) 'number of nonzero singular values = ',p
C
C get natural inverse and inner coord solution....
      call natinv(ut,s,v,n,m,ndim,mdim,p,ginv,sm1,sm1upt)
      call natest(ginv,s,dpr,n,m,p,ndim,mdim,sm1upt,sgn,uinn)
      call covari(s,v,m,mdim,p,qinn,wk,wkar)
      call vnull(v,m,mdim,p,v0)
C
c print general output for all solutions.....
      Call Calcdt(mdim,ndim,m,n,g,uinn,dcalc)
      Call Studres(ndim, n, ut, p, dpr, rpr, srpr)
      Call Prntoc(uot,ndim,n,p,llcsn,d,dcalc,stdev,res,rpr,srpr)
c 
C print inner coordinate solution.....
      If(solopt(1).eq.'y') Then
          header ='Inner Coordinate Solution'
          Call Prtsol(uot,header,nmstus,sta,uinn)
          Call Errlps(qinn,m,mdim,smaj,smin,psi)
          header ='variance of inner coordinate solution'
          Call Prtvar(uot,header,m,sta,smaj,smin,psi)
          Call Vecplt(uplt(1),mdim,nmstus,sta,staloc,uinn,
     +                 smaj,smin,psi)
      EndIf
C
C get and print outer coordinate solution.....
      If(solopt(2).eq.'y') then
          call outercrd(v0,m,mdim,p,alpha,uinn,qinn,uout,qout)
          call errlps(qout,m,mdim,smaj,smin,psi)
          header ='Outer Coordinate Solution'
          Call Prtsol(uot,header,nmstus,sta,uout)
          header ='variance of outer coordinate solution'
          Call Prtvar(uot,header,m,sta,smaj,smin,psi)
          call vecplt(uplt(2),mdim,nmstus,sta,staloc,uout,
     +                 smaj,smin,psi)
      Endif
C
C get and print model coordinate solution.....
      If(solopt(3).eq.'y') then
          call modelcrd(v0,m,mdim,p,uinn,qinn,uapr,qapr,umod,qmod)
          call errlps(qmod,m,mdim,smaj,smin,psi)
          header ='Model Coordinate Solution'
          Call Prtsol(uot,header,nmstus,sta,umod)
          header ='variance of model coordinate solution'
          Call Prtvar(uot,header,m,sta,smaj,smin,psi)
          call vecplt(uplt(3),mdim,nmstus,sta,staloc,umod,
     +                 smaj,smin,psi)
      Endif
C
      STOP
      END
      Subroutine Ginput(uot,upm,uplt,mdim,uapr,alpha,solopt,plotnm)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Get all input except stations and data.                       C
C  Get name of output file, prior model (if any), and type of    C
C  solution (inner, outer, model).                               C
C  Local Variables/Constants:                                    C
C  pmod   = option for prior model (y/n)                         C
C  pfile  = option for prior model file (y/n)                    C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer i, m, n, uot, upm, uplt(3)
      Integer mdim
      Real uapr(mdim), pmodel, alpha
      real xnorth, xeast
      Character*80 ofile, pfname
      Character*1 pmod, pfile
      Character*(*) plotnm(3),  solopt(3)
      Character*8 stanam

C ....Get output file name
      Call getfile2(uot,'Output file name','out','formatted',
     +              ofile)
      Write(uot,6) ofile
    6 Format(10x,'Output file: ',A)
      Write(uot,*)' '


C.....Get solution type(s) and output plot filenames

C.....inner
      Call getopt('Inner coordinate solution?','y/n','n',solopt(1))
      If (solopt(1).eq.'y') Then
         Call getfile2(uplt(1),'Name for inner coord plot file',
     +                      'out','formatted',plotnm(1))
         Write(uot,*)' '
         Write(uot,25)plotnm(1)
   25    Format(10x,'inner coordinate solution plot file: ',A)
      EndIf

C.....outer
      Call getopt('Outer coordinate solution?','y/n','n',solopt(2))
      If (solopt(2).eq.'y') Then
      Print *, ' '      
      Print *, 'Favored direction is alpha degrees counter-clockwise 
     + from North'
      Call askreal('Input alpha ',alpha)
      Call getfile2(uplt(2),'Name for outer coord plot file',
     +                     'out','formatted',plotnm(2))
      Write(uot,30)plotnm(2)
   30 Format(10x,'outer coordinate solution plot file: ',A)
      Write(uot,35)alpha
   35 Format(15x,'alpha = ',f5.1)
      EndIf

C.....model
      Call getopt('Want model coordinate solution?','y/n','n',solopt(3))
      If (solopt(3).eq.'y') Then

      Print *, ' '
      Print *, 'You must have prior model file with format:'
      Print *, '  North component, East component, Station Name '
      Print *, '  Units are the same as data units (ie m, m/yr etc).'
      Print *, '  The order must be the same as order of stations in',
     +         ' station-coordinate file.'

      Call getfile2(upm,'Prior model file name','in',
     +                                     'formatted',pfname)
      Write(uot,10) pfname
   10 Format(10x,'Prior model file name : ',A)

C.....Read the prior model file
        i = 1
   15   Read(upm,100,end=17) xnorth,xeast,stanam
        Write(uot,100) xnorth,xeast,stanam
  100   format(22x,2e10.2,25x,a8)
          uapr(2*(i-1)+1) = xeast
          uapr(2*(i-1)+2) = xnorth
        i = i+1
        Go To 15
   17   Continue
        m = i-1
        Close(unit=upm)

      Call getfile2(uplt(3),'Name for model coord plot file',
     +                     'out','formatted',plotnm(3))
      Write(uot,40)plotnm(3)
   40 Format(10x,'model coordinate solution plot file: ',A)
      EndIf


      RETURN
      END
